#!/usr/bin/perl -w
#
#######################################################
# WebBench
# Written by: Brandon Zehm <caspian@linuxfreak.com>
#
# License:
# This software may be freely distributed, used, and
# modified, provided that this disclaimer and my name
# are included in the code.
#
# Disclaimer:
# -- insert standard disclaimer here ;) --
# I am not responsible for anything you do with this
# software, or anything that happens from the use of
# this software, including but not limited to anything
# you, your lawyers, or anyone else can think of.
#
#######################################################
#
# Purpose: To create a program that is capable of 
#          hitting web pages and benchmarking their 
#          performance.  It should be capable of 
#          receiving and sending cookies and be very 
#          configurable and flexible.
#          
#

use Socket qw (:DEFAULT :crlf);
use Time::HiRes qw ( usleep ualarm gettimeofday tv_interval );
$| = 1;

## Configuration
$conf{'version'}    = "0.20";
$conf{'debug'}      = 1;                  ## Debug level: 1 - 10 (10 being the most debugging information)


## Filename for webbench to use as an Unix socket.
$conf{'socketName'} = "/tmp/webbenchReports.sck";   


## Initialize/Define Global Variables
$conf{'runCount'}         = ""       if (!defined $conf{'runCount'});
$conf{'runTime'}          = ""       if (!defined $conf{'runTime'});
$conf{'clients'}          = 1        if (!defined $conf{'clients'});
$conf{'fastSpawn'}        = 0        if (!defined $conf{'fastSpawn'});
$global{'basicAuth'}      = ""       if (!defined $global{'basicAuth'});
$global{'setCookie'}      = ""       if (!defined $global{'setCookie'});
$global{'totalSent'}      = 0        if (!defined $global{'totalSent'});
$global{'totalPages'}     = 0        if (!defined $global{'totalPages'});
$global{'totalReceived'}  = 0        if (!defined $global{'totalReceived'});
$global{'lstPagesSec'}    = 0        if (!defined $global{'lstPagesSec'});
$global{'avgPagesSec'}    = 0        if (!defined $global{'avgPagesSec'});
$global{'seconds'}        = 0        if (!defined $global{'seconds'});
$global{'loop'}           = 0        if (!defined $global{'loop'});
$global{'tmp'}            = ""       if (!defined $global{'tmp'});

quit("Error: \$conf{'script'} must be defined or running this is pointless!  Exiting.")    if (!defined $conf{'script'});



## Intercept all signals and exit cleanly ###
$SIG{'QUIT'} = \&sigQuit;
$SIG{'INT'} = \&sigQuit;
$SIG{'KILL'} = \&sigQuit;
$SIG{'TERM'} = \&sigQuit;
$SIG{'HUP'} = \&sigQuit;

## Include all the nessacary functions.
do "functions.inc";









################################
##  START THE REPORTS SERVER  ##
################################
unless (fork) {
  
  print "$$ - Starting reports server.\n";
  
  my %local;
  
  ## Open a unix socket for clients to connect to.
  $SIG{CHLD} = \&reaper;
  
  bindSocket($conf{'socketName'}) || quit("Reports Server there was an error binding to a unix socket.");
  
  listen(SOCKET,200) || quit("Reports Server: There was an error with listen()");
  
  while (accept(CLIENT,SOCKET)) {
    while (<CLIENT>) {
      (
        $local{'inPages'}, 
        $local{'inSent'}, 
        $local{'inReceived'},
        $local{'inFinished'}
      ) = split (/:/, $_);
      
      $global{'totalPages'}      +=  $local{'inPages'};
      $global{'totalSent'}       +=  $local{'inSent'};
      $global{'totalReceived'}   +=  $local{'inReceived'};
      $global{'clientsFinished'} +=  $local{'inFinished'};
    }
    close CLIENT;
    
    
    ## Print Report if needed
    if ($global{'seconds'} != time()) {
      $global{'seconds'} = time();
      printReport();
    }
    
    
    ## If all the clients are finished print a report and exit.
    if ($global{'clientsFinished'} >= $conf{'clients'}) {
      print "--------------------= Summary =--------------------";
      printReport();
      ## Close and remove unix socket.
      closeSocket();
      system("rm -f '$conf{'socketName'}'");
      quit("\nExiting:  All clients are finished.\n");
    }
    
  }
  quit ("Reports Server Exiting: I shouldn't ever exit here!");
}










#####################
##  SPAWN CLIENTS  ##
#####################
for ($global{'loop'} = 1; $global{'loop'} <= $conf{'clients'}; $global{'loop'} += 1) {
    
  print "$$ - Spawning child: $global{'loop'}\n" if ($conf{'debug'} > 3);
  
  ## Fork the process (creating a child)  
  unless (fork) {
    
    ##  Main client loop  ##
    
    ## If testing by count
    if ($conf{'runCount'} ne "") {
      for ($global{'loop'} = 1; $global{'loop'} <= $conf{'runCount'}; $global{'loop'} += 1) {
        eval $conf{'script'};
        if ($global{'seconds'} != time) {
          $global{'seconds'} = time;
          sendReport(0);
        }
      }
    }
    
    ## Else if testing by time
    elsif ($conf{'runTime'} ne "") {
      while ( (time - $^T) < $conf{'runTime'}) {
        eval $conf{'script'};
        if ($global{'seconds'} != time) {
          $global{'seconds'} = time;
          sendReport(0);
        }
      }
    }
    
    
    
    ## Send Final Report and Exit
    sendReport(1);
    print "$$ - I'm done with my work! Exiting\n" if ($conf{'debug'} > 3);
    quit("");
  }
  
  ## Sleep between 0 and 1 seconds before spawning another one.
  unless($conf{'fastSpawn'}) {
    $val = rand 1000000;
    usleep($val);
  }
}






## Exit
quit();


